@@ -39,7 +39,8 @@ FILE	*in_dat;
 char	SendBack;
 char	Connected;
 char	DVwait;
-unsigned char	PacketIdSave[2];
+unsigned char	RcvPacketIdSave[2];
+unsigned char	SndPacketIdSave[2];
 unsigned int	crc;
 unsigned int	Packet_ID;
 
@@ -72,6 +73,7 @@ void	voice_send(void)
                 	ConSend();
                 	ConnectWait = TRUE;
 			AckTimeOut = 0;
+			RelaySW = FALSE;
 			DSTREQcnt = 0;
 			NakCount = 0;
 			Connected = FALSE;
@@ -175,7 +177,6 @@ void	voice_frame_send(void)
 		{
 			ConnectWait = FALSE;
 			FirstHeaderSend = FALSE;
-			RelaySW = FALSE;
 			Connected = FALSE;
 		}
                	timeradd (&t1, &t4, &t3);
@@ -202,10 +203,10 @@ void    ConSend(void)
         sndAddr.sin_port = db_port;
         sndAddr.sin_addr = dbIP;
 	Packet_ID = rand();
-	PacketIdSave[0] = (Packet_ID >> 8) & 0xff;
-	PacketIdSave[1] = Packet_ID & 0xff;
-	send_msg.PacketID[0] = PacketIdSave[0];
-	send_msg.PacketID[1] = PacketIdSave[1];
+	SndPacketIdSave[0] = (Packet_ID >> 8) & 0xff;
+	SndPacketIdSave[1] = Packet_ID & 0xff;
+	send_msg.PacketID[0] = SndPacketIdSave[0];
+	send_msg.PacketID[1] = SndPacketIdSave[1];
 	memcpy (&send_msg.srcCall, &client_call, 8);
 	memcpy (&send_msg.dstCall, &CallerCall, 8);
 	memcpy (&send_msg.Rep1Call, &client_call, 8);
@@ -232,13 +233,13 @@ void	AckSend(char srcRptCall[8], char PacketID[2])
 	return;
 }
 
-void	ReqSendSender (void)
+void    ReqSendSender()
 {
-	sndAddr.sin_family = PF_INET;
+        sndAddr.sin_family = PF_INET;
         sndAddr.sin_port = dstPort;
         sndAddr.sin_addr = dstIP;
-        sendto (udpSocket, "\0\0\0\0SRCREQ UDP hole punching", 28, 0,
-                 (struct sockaddr *)&sndAddr, sizeof(sndAddr));
+        sendto (udpSocket, "\0\0\0\0DSTREQ UDP hole punching", 28, 0,
+                (struct sockaddr *)&sndAddr, sizeof(sndAddr));
 }
 
 int main(int argc, const char **argv){
@@ -258,10 +259,10 @@ int main(int argc, const char **argv){
 	struct sockaddr_in dstAddr;
 	//struct	repMsg	reqConMsg;
 
-	//struct sockaddr_in addr;
  	struct hostent *hp;
   	char   buf[256];
 	int    numrcv;
+	int	cmdin;
 
 	FILE	*dat;
 
@@ -272,16 +273,35 @@ int main(int argc, const char **argv){
 
 	config (argc, (char **)argv);
 
+        if (cmd_mode == 0)
+        {
+                cmdin = 0;
+        }
+        else
+        {
+                mkfifo ("/tmp/dv_echo-cmdin", 0600);
+
+                cmdin = open ("/tmp/dv_echo-cmdin", O_RDONLY | O_NONBLOCK);
+                if (cmdin < 0)
+                {
+                        logfile = fopen (LOGFILE, "a");
+                        now = time(NULL);
+                        fprintf (logfile, "%24.24s ERROR FIFO file not open /tmp/dv_echo-cmdin\n", ctime(&now));
+                        fclose (logfile);
+                        cmdin = 0;
+                }
+        }
+
 	//sockaddr_in 構造体のセット
 	bzero((char *)&dstAddr, sizeof(dstAddr));
 	dstAddr.sin_family = PF_INET;
 	dstAddr.sin_port = trust_port;
   
 	hp = gethostbyname(TrustDomainName);
-	bcopy(hp->h_addr, &dstAddr.sin_addr, hp->h_length);
+	memcpy (&dstAddr.sin_addr, hp->h_addr_list[0], hp->h_length);
 
         hp = gethostbyname(TrustDbDomainName);
-        bcopy(hp->h_addr, &dbIP, hp->h_length);
+        memcpy (&dbIP, hp->h_addr_list[0], hp->h_length);
 
 	//ソケットの生成
 	dstSocket = socket(PF_INET, SOCK_STREAM, 0);
@@ -290,19 +310,23 @@ int main(int argc, const char **argv){
 	//接続
 	if (connect(dstSocket, (struct sockaddr *)&dstAddr, 
 			sizeof(dstAddr)) < 0){
-	printf("Does not connect to '%s'.\n",TrustDomainName);
+		logfile = fopen (LOGFILE, "a");
+		now = time(NULL);
+		fprintf(logfile, "%24.24s Does not connect to '%s'.\n",ctime(&now), TrustDomainName);
+		fclose (logfile);
 	return(-1);
 	}
 
-	printf("Connect to '%s'.\n", TrustDomainName);
+	logfile = fopen (LOGFILE, "a");
+	now = time(NULL);
+	fprintf(logfile, "%24.24s Connect to '%s'.\n", ctime(&now), TrustDomainName);
+	fclose (logfile);
 
 	//コールサイン入力のプロンプト
 	memset (buf, 0x20, 14); 
 	while (memcmp (buf,"Repeater Call:", 14) != 0)
 	{
 		numrcv = recv(dstSocket, buf, 64, 0);
-		buf[numrcv] = 0x00;
-		printf("%s\n",buf);
 	}
 
 	//コールサインの送信
@@ -317,8 +341,6 @@ int main(int argc, const char **argv){
 	while (memcmp (buf, "Password:", 9) != 0)
 	{
         	numrcv = recv(dstSocket, buf,  255, 0);
-        	buf[numrcv] = 0x00;
-		printf ("%s\n",buf);
 	}
 
         //パスワードの送信
@@ -327,14 +349,12 @@ int main(int argc, const char **argv){
         //ポート番号入力のプロンプト
 	memset (buf, 0x20, 12);
 	numrcv = recv(dstSocket, buf, 255, 0);
-        if (memcmp (buf, "Port Number:", 12) == 0)
+        if (!memcmp (buf, "Port Number:", 12) == 0)
 	{
-       	 	buf[numrcv] = 0x00;
-        	printf("%s\n",buf);
-	}
-	else 
-	{
-		printf ("Invalide CallSign or Password\n");
+		logfile = fopen(LOGFILE, "a");
+		now = time(NULL);
+		fprintf (logfile, "%24.24s Invalide CallSign or Password\n", ctime(&now));
+		fclose (logfile);
 		CloseSocket();
 	}
 
@@ -351,9 +371,10 @@ int main(int argc, const char **argv){
         FD_ZERO(&Mask);
         FD_SET(dstSocket, &Mask);
 	FD_SET(udpSocket, &Mask);
-        FD_SET(0,&Mask);
+        FD_SET(cmdin,&Mask);
         width = dstSocket;
-	if (dstSocket < udpSocket) width = udpSocket;
+	if (width < udpSocket) width = udpSocket;
+	if (width < cmdin) width = cmdin;
 	width++;
 	Send_Connect = FALSE;
 	Recv_Connect = FALSE;
@@ -391,7 +412,10 @@ int main(int argc, const char **argv){
                                                 ConnectWait = FALSE;
                                                 FirstHeaderSend = TRUE;
 						AckTimeOut = 0;
-                                                printf ("Change to Relay Mode\n");
+						logfile = fopen (LOGFILE, "a");
+						now = time(NULL);
+                                                fprintf (logfile, "%24.24s Change to Relay Mode\n", ctime(&now));
+						fclose (logfile);
                                         }
 				}
 				if (DVwait)
@@ -415,7 +439,7 @@ int main(int argc, const char **argv){
                                         }
                         		if (reqConMsg.reqType == Ack)
                         		{
-						if (!Connected && !memcmp(&PacketIdSave, &reqConMsg.ackConnect.PacketID, 2))
+						if (!Connected && !memcmp(&SndPacketIdSave, &reqConMsg.ackConnect.PacketID, 2))
 						{
                                                		dstPort = reqConMsg.ackConnect.port;
                                                 	dstIP = reqConMsg.ackConnect.ip;
@@ -429,10 +453,6 @@ int main(int argc, const char **argv){
                         		}
                                         if (reqConMsg.reqType == Nak)
                                         {
-                                                printf ("Nak %2.2x %8.8s %s %d\n",
-                                                        reqConMsg.reqType, reqConMsg.ackConnect.YourCall,
-                                                        inet_ntoa (reqConMsg.ackConnect.ip),
-                                                        ntohs(reqConMsg.ackConnect.port));
 						usleep(500000);
 						ConSend();
 						AckTimeOut = 0;
@@ -445,27 +465,16 @@ int main(int argc, const char **argv){
                                         }
                                         else if (reqConMsg.reqType == ReqConnect)
                                         {
-                                                printf ("Req %2.2x %2.2x %8.8s %8.8s %s %d\n",
-							reqConMsg.reqType,
-                                                        reqConMsg.reqConnect.protocol,
-                                                        reqConMsg.reqConnect.YourCall,
-							reqConMsg.reqConnect.Rpt1Call,
-                                                        inet_ntoa(reqConMsg.reqConnect.ip),
-                                                        ntohs(reqConMsg.reqConnect.port));
-						PacketIdSave[0] = reqConMsg.reqConnect.PacketID[0];
-						PacketIdSave[1] = reqConMsg.reqConnect.PacketID[1];
+						RcvPacketIdSave[0] = reqConMsg.reqConnect.PacketID[0];
+						RcvPacketIdSave[1] = reqConMsg.reqConnect.PacketID[1];
 						ReqSendSender();
 						DSTREQcnt = 0;
-						//dstIP = reqConMsg.reqConnect.ip;
-						//dstPort = reqConMsg.reqConnect.port;
-						//if (AckSend(reqConMsg.reqConnect.Rpt1Call) == Ack)
-						//			ReqSendSender();
-						AckSend(reqConMsg.reqConnect.Rpt1Call, reqConMsg.reqConnect.PacketID);
+						AckSend(reqConMsg.reqConnect.Rpt1Call, RcvPacketIdSave);
                                         }
 				}
-				if(FD_ISSET(0, &readOk))
+				if(FD_ISSET(cmdin, &readOk))
 				{
-                			fgets(buf, 255, stdin);
+                			read (cmdin, buf, 255);
                 			if (memcmp (buf, "end", 3) == 0)
 					{
 						reqConMsg.reqType = ReqEnd;
@@ -516,7 +525,7 @@ int main(int argc, const char **argv){
 						}
         				}
 					if (!memcmp (send_packet.ID, "DSVT", 4)
-						&& !memcmp(PacketIdSave, send_packet.B_header.PacketID, 2))
+						&& !memcmp(RcvPacketIdSave, send_packet.B_header.PacketID, 2))
 					{
 						DVwait = FALSE;
 						if (!dat) dat = fopen ("/tmp/dv_echo.bin", "wb");
